"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Logger_1 = require("./Logger");
var LoggerFactoryService_1 = require("./LoggerFactoryService");
var ST = require("stacktrace-js");
var Message = (function () {
    function Message(ready, message) {
        this._ready = ready;
        this._message = message;
    }
    Object.defineProperty(Message.prototype, "ready", {
        get: function () {
            return this._ready;
        },
        set: function (value) {
            this._ready = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Message.prototype, "message", {
        get: function () {
            return this._message;
        },
        set: function (value) {
            this._message = value;
        },
        enumerable: true,
        configurable: true
    });
    return Message;
}());
var LinkedNode = (function () {
    function LinkedNode(value) {
        this._previous = null;
        this._next = null;
        this._value = value;
    }
    Object.defineProperty(LinkedNode.prototype, "previous", {
        get: function () {
            return this._previous;
        },
        set: function (value) {
            this._previous = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LinkedNode.prototype, "next", {
        get: function () {
            return this._next;
        },
        set: function (value) {
            this._next = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LinkedNode.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    return LinkedNode;
}());
var LinkedList = (function () {
    function LinkedList() {
        this.head = null;
        this.size = 0;
    }
    LinkedList.prototype.addHead = function (value) {
        if (!this.createHeadIfNeeded(value)) {
            var nextNode = this.head.next;
            var newHeadNode = new LinkedNode(value);
            if (nextNode != null) {
                nextNode.previous = newHeadNode;
                newHeadNode.next = nextNode;
            }
            this.head = newHeadNode;
        }
        this.size++;
    };
    LinkedList.prototype.addTail = function (value) {
        if (!this.createHeadIfNeeded(value)) {
            var oldTailNode = this.getTailNode();
            var newTailNode = new LinkedNode(value);
            oldTailNode.next = newTailNode;
            newTailNode.previous = oldTailNode;
        }
        this.size++;
    };
    LinkedList.prototype.clear = function () {
        this.head = null;
        this.size = 0;
    };
    LinkedList.prototype.getHead = function () {
        if (this.head != null) {
            return this.head.value;
        }
        return null;
    };
    LinkedList.prototype.removeHead = function () {
        if (this.head != null) {
            var oldHead = this.head;
            var value = oldHead.value;
            this.head = oldHead.next;
            this.size--;
            return value;
        }
        return null;
    };
    LinkedList.prototype.getTail = function () {
        var node = this.getTailNode();
        if (node != null) {
            return node.value;
        }
        return null;
    };
    LinkedList.prototype.removeTail = function () {
        var node = this.getTailNode();
        if (node != null) {
            if (node === this.head) {
                this.head = null;
            }
            else {
                var previousNode = node.previous;
                previousNode.next = null;
            }
            this.size--;
            return node.value;
        }
        return null;
    };
    LinkedList.prototype.getSize = function () {
        return this.size;
    };
    LinkedList.prototype.createHeadIfNeeded = function (value) {
        if (this.head == null) {
            this.head = new LinkedNode(value);
            return true;
        }
        return false;
    };
    LinkedList.prototype.getTailNode = function () {
        if (this.head == null) {
            return null;
        }
        var node = this.head;
        while (node.next != null) {
            node = node.next;
        }
        return node;
    };
    return LinkedList;
}());
/**
 * Abstract base logger, extend to easily implement a custom logger that
 * logs wherever you want. You only need to implement doLog(msg: string) and
 * log that somewhere (it will contain format and everything else).
 */
var AbstractLogger = (function () {
    function AbstractLogger(name, rule) {
        this.open = true;
        this._allMessages = new LinkedList();
        this.name = name;
        this.rule = rule;
        this.level = rule.level;
    }
    AbstractLogger.prototype.trace = function (msg, error) {
        this.log(Logger_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debug = function (msg, error) {
        this.log(Logger_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.info = function (msg, error) {
        this.log(Logger_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warn = function (msg, error) {
        this.log(Logger_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.error = function (msg, error) {
        this.log(Logger_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatal = function (msg, error) {
        this.log(Logger_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.tracec = function (msg, error) {
        this.logc(Logger_1.LogLevel.Trace, msg, error);
    };
    AbstractLogger.prototype.debugc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Debug, msg, error);
    };
    AbstractLogger.prototype.infoc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Info, msg, error);
    };
    AbstractLogger.prototype.warnc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Warn, msg, error);
    };
    AbstractLogger.prototype.errorc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Error, msg, error);
    };
    AbstractLogger.prototype.fatalc = function (msg, error) {
        this.logc(Logger_1.LogLevel.Fatal, msg, error);
    };
    AbstractLogger.prototype.isTraceEnabled = function () {
        return this.level == Logger_1.LogLevel.Trace;
    };
    AbstractLogger.prototype.isDebugEnabled = function () {
        return this.level <= Logger_1.LogLevel.Debug;
    };
    AbstractLogger.prototype.isInfoEnabled = function () {
        return this.level <= Logger_1.LogLevel.Info;
    };
    AbstractLogger.prototype.isWarnEnabled = function () {
        return this.level <= Logger_1.LogLevel.Warn;
    };
    AbstractLogger.prototype.isErrorEnabled = function () {
        return this.level <= Logger_1.LogLevel.Error;
    };
    AbstractLogger.prototype.isFatalEnabled = function () {
        return this.level <= Logger_1.LogLevel.Fatal;
    };
    AbstractLogger.prototype.getLogLevel = function () {
        return this.level;
    };
    AbstractLogger.prototype.log = function (level, msg, error) {
        if (this.open && this.level <= level) {
            this._allMessages.addTail(this.createMessage(level, msg, new Date(), error));
            this.processMessages();
        }
    };
    AbstractLogger.prototype.logc = function (level, msg, error) {
        if (this.open && this.level <= level) {
            this._allMessages.addTail(this.createMessage(level, msg(), new Date(), error !== undefined ? error() : undefined));
            this.processMessages();
        }
    };
    AbstractLogger.prototype.isOpen = function () {
        return this.open;
    };
    AbstractLogger.prototype.close = function () {
        this.open = false;
        this._allMessages.clear();
    };
    AbstractLogger.prototype.createMessage = function (level, msg, date, error) {
        var _this = this;
        var lpad = function (value, chars, padWith) {
            var howMany = chars - value.length;
            if (howMany > 0) {
                var res = '';
                for (var i = 0; i < howMany; i++) {
                    res += padWith;
                }
                res += value;
                return res;
            }
            return value;
        };
        var fullYear = function (date) {
            return lpad(date.getFullYear().toString(), 4, '0');
        };
        var month = function (date) {
            return lpad((date.getMonth() + 1).toString(), 2, '0');
        };
        var day = function (date) {
            return lpad(date.getDate().toString(), 2, '0');
        };
        var hours = function (date) {
            return lpad(date.getHours().toString(), 2, '0');
        };
        var minutes = function (date) {
            return lpad(date.getMinutes().toString(), 2, '0');
        };
        var seconds = function (date) {
            return lpad(date.getSeconds().toString(), 2, '0');
        };
        var millis = function (date) {
            return date.getMilliseconds().toString();
        };
        var result = "";
        if (this.rule.logFormat.showTimeStamp) {
            var dateSeparator = this.rule.logFormat.dateFormat.dateSeparator;
            var ds = '';
            switch (this.rule.logFormat.dateFormat.formatEnum) {
                case LoggerFactoryService_1.DateFormatEnum.Default:
                    // yyyy-mm-dd hh:mm:ss,m
                    ds = fullYear(date) + dateSeparator + month(date) + dateSeparator + day(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date) + "," + millis(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearMonthDayTime:
                    ds = fullYear(date) + dateSeparator + month(date) + dateSeparator + day(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearDayMonthWithFullTime:
                    ds = fullYear(date) + dateSeparator + day(date) + dateSeparator + month(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date) + "," + millis(date);
                    break;
                case LoggerFactoryService_1.DateFormatEnum.YearDayMonthTime:
                    ds = fullYear(date) + dateSeparator + day(date) + dateSeparator + month(date) + ' ' +
                        hours(date) + ':' + minutes(date) + ":" + seconds(date);
                    break;
                default:
                    throw new Error("Unsupported date format enum: " + this.rule.logFormat.dateFormat.formatEnum);
            }
            result += ds + " ";
        }
        result += Logger_1.LogLevel[level].toUpperCase() + " ";
        if (this.rule.logFormat.showLoggerName) {
            result += "[" + this.name + "]";
        }
        result += ' ' + msg;
        if (error !== undefined) {
            var message_1 = new Message(false);
            result += '\n' + error.name + ": " + error.message + "\n@";
            ST.fromError(error, { offline: true }).then(function (frames) {
                var stackStr = (frames.map(function (frame) {
                    return frame.toString();
                })).join('\n  ');
                result += '\n' + stackStr;
                message_1.message = result;
                message_1.ready = true;
                _this.processMessages();
            });
            return message_1;
        }
        else {
            return new Message(true, result);
        }
    };
    AbstractLogger.prototype.processMessages = function () {
        // Basically we wait until errors are resolved (those messages
        // may not be ready).
        var msgs = this._allMessages;
        if (msgs.getSize() > 0) {
            do {
                var msg = msgs.getHead();
                if (msg != null) {
                    if (!msg.ready) {
                        break;
                    }
                    msgs.removeHead();
                    this.doLog(msg.message);
                }
            } while (msgs.getSize() > 0);
        }
    };
    return AbstractLogger;
}());
exports.AbstractLogger = AbstractLogger;
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw exception.
 */
var ConsoleLoggerImpl = (function (_super) {
    __extends(ConsoleLoggerImpl, _super);
    function ConsoleLoggerImpl(name, rule) {
        _super.call(this, name, rule);
    }
    ConsoleLoggerImpl.prototype.doLog = function (msg) {
        if (console !== undefined) {
            console.log(msg);
        }
        else {
            throw new Error("Console is not defined, cannot log msg: " + msg);
        }
    };
    return ConsoleLoggerImpl;
}(AbstractLogger));
exports.ConsoleLoggerImpl = ConsoleLoggerImpl;
/**
 * Logger which buffers all messages, use with care due to possible high memory footprint.
 * Can be convenient in some cases. Call toString() for full output, or cast to this class
 * and call getMessages() to do something with it yourself.
 */
var MessageBufferLoggerImpl = (function (_super) {
    __extends(MessageBufferLoggerImpl, _super);
    function MessageBufferLoggerImpl(name, rule) {
        _super.call(this, name, rule);
        this.messages = [];
    }
    MessageBufferLoggerImpl.prototype.doLog = function (msg) {
        this.messages.push(msg);
    };
    MessageBufferLoggerImpl.prototype.close = function () {
        this.messages = [];
        _super.prototype.close.call(this);
    };
    MessageBufferLoggerImpl.prototype.getMessages = function () {
        return this.messages;
    };
    MessageBufferLoggerImpl.prototype.toString = function () {
        return this.messages.map(function (msg) {
            return msg;
        }).join("\n");
    };
    return MessageBufferLoggerImpl;
}(AbstractLogger));
exports.MessageBufferLoggerImpl = MessageBufferLoggerImpl;
//# sourceMappingURL=LoggerImpl.js.map